{% extends 'layout.html.jinja' %}

{% block body_class %}single single-page{% endblock %}
{% block title %}{{ page.title }}{% endblock %}
{% block description %}{{ page.description }}{% endblock %}

{% block content %}
	<article class="page">
		<header class="page-header">
			<h2 class="page-title">
				{{ page.title }}
			{%- if page.subtitle %}
				<small>{{ page.subtitle }}</small>
			{%- endif %}
			</h2>
		</header>
		<div class="page-body">
			{{ page.body }}
		</div>
	</article>
{% endblock %}
